import { forwardRef, useState, useRef, useEffect } from "react";
import styled from "styled-components";
import { useCounter, useCallbackRef, useForkRef, useForkHandler } from "../hooks";
import { MoveHorizontalIcon } from "../icons";
import { Input } from "./input";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const FLOATING_POINT_REGEX = /^[Ee0-9+\-.]$/;
function sanitize(value) {
  return value.split("").filter(char => FLOATING_POINT_REGEX.test(char)).join("");
}
function isNumericKeyboardEvent(event) {
  if (!event.key || event.ctrlKey || event.altKey || event.metaKey || event.key.length !== 1) {
    return true;
  }
  return FLOATING_POINT_REGEX.test(event.key);
}
function getStepFactor(event) {
  if (event.metaKey || event.ctrlKey) {
    return 0.1;
  }
  if (event.shiftKey) {
    return 10;
  }
  return 1;
}
function useDragger(_ref) {
  let {
    onDragChange,
    onUpdate
  } = _ref;
  const ref = useRef();
  const savedCallback = useCallbackRef(onUpdate);
  const [active, setActive] = useState(false);
  const start = () => {
    setActive(true);
    onDragChange?.(true);
  };
  useEffect(() => {
    if (active) {
      ref.current?.focus();
      ref.current?.requestPointerLock();
      const onRelease = () => {
        setActive(false);
        onDragChange?.(false);
      };
      const onPointerLockChange = () => {
        if (document.pointerLockElement === ref.current) {
          document.addEventListener("mousemove", savedCallback, false);
        } else {
          setActive(false);
          onDragChange?.(false);
        }
      };
      document.addEventListener("mouseup", onRelease);
      document.addEventListener("pointerlockchange", onPointerLockChange, false);
      return () => {
        document.removeEventListener("mouseup", onRelease);
        document.removeEventListener("pointerlockchange", onPointerLockChange, false);
        document.removeEventListener("mousemove", savedCallback, false);
        document.exitPointerLock?.();
      };
    }
    return;
  }, [active]);
  return {
    ref,
    active,
    start
  };
}
const StyledSuffix = styled(Input.Suffix).withConfig({
  displayName: "number-input__StyledSuffix",
  componentId: "sc-q47sci-0"
})(["visibility:visible;"]);
const DragHandle = styled(Input.Suffix).withConfig({
  displayName: "number-input__DragHandle",
  componentId: "sc-q47sci-1"
})(["color:", ";visibility:hidden;pointer-events:auto;cursor:ew-resize;"], p => p.active ? p.theme.colors.primary.base : p.theme.colors.text);
const StyledGroup = styled(Input.Group).withConfig({
  displayName: "number-input__StyledGroup",
  componentId: "sc-q47sci-2"
})(["input:focus:enabled{padding-right:4px;& ~ ", "{display:none;}}&:hover input:enabled{&:focus{padding-right:24px;}& ~ ", "{visibility:hidden;}& ~ ", "{visibility:visible;}}"], StyledSuffix, StyledSuffix, DragHandle);
export const NumberInput = /*#__PURE__*/forwardRef((props, ref) => {
  const {
    className,
    suffix,
    min = Number.MIN_SAFE_INTEGER,
    max = Number.MAX_SAFE_INTEGER,
    step = 1,
    isInvalid,
    readOnly,
    disabled,
    onDragChange
  } = props;
  const counter = useCounter(props);
  const dragger = useDragger({
    onDragChange,
    onUpdate(event) {
      const dx = event.movementX;
      if (dx !== 0) {
        counter.increment(dx * step);
      }
    }
  });
  const handleChange = val => {
    counter.update(sanitize(val));
  };
  const handleBlur = () => {
    counter.cast(counter.value);
  };
  const handleKeyDown = event => {
    if (!isNumericKeyboardEvent(event)) {
      event.preventDefault();
    }
    if (readOnly || disabled) {
      return;
    }
    const stepFactor = getStepFactor(event) * step;
    const keyMap = {
      ArrowUp: () => counter.increment(stepFactor),
      ArrowDown: () => counter.decrement(stepFactor),
      Home: () => counter.update(min),
      End: () => counter.update(max)
    };
    const action = keyMap[event.key];
    if (action) {
      event.preventDefault();
      action();
    }
  };
  const inputProps = {
    ref: useForkRef(ref, dragger.ref),
    id: props.id,
    onFocus: props.onFocus,
    onBlur: useForkHandler(props.onBlur, handleBlur),
    onKeyDown: useForkHandler(props.onKeyDown, handleKeyDown)
  };
  return /*#__PURE__*/_jsxs(StyledGroup, {
    className: className,
    children: [/*#__PURE__*/_jsx(Input, {
      value: counter.value,
      onChange: handleChange,
      isInvalid: isInvalid,
      readOnly: readOnly,
      disabled: disabled,
      ...inputProps
    }), /*#__PURE__*/_jsx(DragHandle, {
      active: dragger.active,
      onMouseDown: dragger.start,
      children: /*#__PURE__*/_jsx(MoveHorizontalIcon, {})
    }), suffix ? /*#__PURE__*/_jsx(StyledSuffix, {
      children: suffix
    }) : null]
  });
});
if (process.env.NODE_ENV !== "production") {
  NumberInput.displayName = "NumberInput";
}