import * as ToastPrimitive from "@radix-ui/react-toast";
import { createContext, useState, useMemo, useEffect, useContext } from "react";
import styled, { css, keyframes } from "styled-components";
import { nanoid } from "nanoid";
import { Box } from "./box";
import { Portal } from "./portal";
import { IconButton } from "./icon-button";
import { CloseIcon, InfoIcon, CheckIcon, ErrorIcon, WarningIcon } from "../icons";
import { useCallbackRef } from "../hooks";
import { omit } from "../utils/object-util";
import { jsx as _jsx } from "react/jsx-runtime";
import { jsxs as _jsxs } from "react/jsx-runtime";
const ToastContext = /*#__PURE__*/createContext(null);
export function useToast() {
  return useContext(ToastContext);
}
const hide = keyframes(["from{margin-bottom:0;opacity:1;}to{margin-bottom:-16px;opacity:0;}"]);
const slideIn = keyframes(["from{transform:translateX(calc(100% + var(--viewport-padding)));}to{transform:translateX(0);}"]);
const swipeOut = keyframes(["from{transform:translateX(var(--radix-toast-swipe-end-x));}to{transform:translateX(calc(100% + var(--viewport-padding)));}"]);
const makeTypeColor = (type, color) => p => p.type === type && css(["color:", ";"], p.theme.colors[color].base);
const ToastIcon = styled.span.withConfig({
  displayName: "toast__ToastIcon",
  componentId: "sc-xg65pk-0"
})(["display:inline-flex;font-size:", "px;", " ", " ", " ", ""], p => p.theme.fontSizes[5], makeTypeColor("info", "info"), makeTypeColor("success", "success"), makeTypeColor("error", "danger"), makeTypeColor("warning", "warning"));
const ToastTitle = styled(ToastPrimitive.Title).withConfig({
  displayName: "toast__ToastTitle",
  componentId: "sc-xg65pk-1"
})(["color:", ";font-size:", "px;"], p => p.theme.colors.heading, p => p.theme.fontSizes.lg);
const ToastDescription = styled(ToastPrimitive.Description).withConfig({
  displayName: "toast__ToastDescription",
  componentId: "sc-xg65pk-2"
})(["font-size:", "px;"], p => p.theme.fontSizes.md);
const ToastAction = styled(ToastPrimitive.Action).withConfig({
  displayName: "toast__ToastAction",
  componentId: "sc-xg65pk-3"
})(["flex-shrink:0;"]);
const ToastClose = styled(ToastPrimitive.Close).withConfig({
  displayName: "toast__ToastClose",
  componentId: "sc-xg65pk-4"
})(["flex-shrink:0;"]);
const ToastRoot = styled(ToastPrimitive.Root).withConfig({
  displayName: "toast__ToastRoot",
  componentId: "sc-xg65pk-5"
})(["display:flex;align-items:center;column-gap:", "px;padding:", "px;border-radius:", "px;box-shadow:", ";background:", ";line-height:", ";&[data-state=\"open\"]{animation:", " 150ms ", ";}&[data-state=\"closed\"]{animation:", " 100ms ", ";}&[data-swipe=\"move\"]{transform:translateX(var(--radix-toast-swipe-move-x));}&[data-swipe=\"cancel\"]{transform:translateX(0);transition:transform 200ms ", ";}&[data-swipe=\"end\"]{animation:", " 100ms ", ";}"], p => p.theme.space.md, p => p.theme.space.md, p => p.theme.radii.base, p => p.theme.shadows.base, p => p.theme.colors.popover.bg, p => p.theme.lineHeights.base, slideIn, p => p.theme.transitions.easeOut, hide, p => p.theme.transitions.easeIn, p => p.theme.transitions.easeOut, swipeOut, p => p.theme.transitions.easeOut);
const ToastViewport = styled(ToastPrimitive.Viewport).withConfig({
  displayName: "toast__ToastViewport",
  componentId: "sc-xg65pk-6"
})(["--viewport-padding:16px;position:fixed;bottom:8px;right:0;display:flex;flex-direction:column;padding:var(--viewport-padding);gap:16px;width:420px;max-width:100vw;margin:0;list-style:none;z-index:1010;outline:none;"]);
const iconMap = {
  info: /*#__PURE__*/_jsx(InfoIcon, {}),
  success: /*#__PURE__*/_jsx(CheckIcon, {}),
  error: /*#__PURE__*/_jsx(ErrorIcon, {}),
  warning: /*#__PURE__*/_jsx(WarningIcon, {})
};
function Toast(_ref) {
  let {
    title,
    description,
    icon,
    action,
    actionAltText = "Action",
    duration = 4000,
    type,
    sensitivity = "foreground",
    onClose
  } = _ref;
  const [open, setOpen] = useState(true);
  const iconNode = icon ?? (type ? iconMap[type] : null);
  useEffect(() => {
    if (!open) {
      const id = setTimeout(() => onClose?.(), 200);
      return () => clearTimeout(id);
    }
    return;
  }, [open]);
  return /*#__PURE__*/_jsxs(ToastRoot, {
    open: open,
    onOpenChange: setOpen,
    type: sensitivity,
    duration: duration,
    children: [iconNode ? /*#__PURE__*/_jsx(ToastIcon, {
      type: type,
      children: iconNode
    }) : null, /*#__PURE__*/_jsxs(Box, {
      flexGrow: 1,
      children: [/*#__PURE__*/_jsx(ToastTitle, {
        children: title
      }), description ? /*#__PURE__*/_jsx(ToastDescription, {
        children: description
      }) : null]
    }), action ? /*#__PURE__*/_jsx(ToastAction, {
      asChild: true,
      altText: actionAltText,
      children: action
    }) : null, /*#__PURE__*/_jsx(ToastClose, {
      asChild: true,
      children: /*#__PURE__*/_jsx(IconButton, {
        type: "button",
        variant: "text",
        "aria-label": "Close",
        icon: /*#__PURE__*/_jsx(CloseIcon, {
          size: 24
        })
      })
    })]
  });
}
export function ToastProvider(_ref2) {
  let {
    container,
    children
  } = _ref2;
  const [toasts, setToasts] = useState({});
  const open = useCallbackRef(props => {
    const key = props.key ?? nanoid();
    setToasts(t => {
      return {
        ...t,
        [key]: {
          ...t[key],
          ...props,
          key
        }
      };
    });
    return key;
  });
  const close = useCallbackRef(key => {
    if (toasts[key]) {
      setToasts(t => {
        return {
          ...t,
          [key]: {
            ...t[key],
            duration: 1
          }
        };
      });
    }
  });
  const info = useCallbackRef(props => {
    return open({
      ...props,
      type: "info"
    });
  });
  const success = useCallbackRef(props => {
    return open({
      ...props,
      type: "success"
    });
  });
  const error = useCallbackRef(props => {
    return open({
      ...props,
      type: "error"
    });
  });
  const warning = useCallbackRef(props => {
    return open({
      ...props,
      type: "warning"
    });
  });
  const contextValue = useMemo(() => {
    return {
      open,
      close,
      info,
      success,
      error,
      warning
    };
  }, []);
  const handleClose = useCallbackRef(key => {
    setToasts(t => omit(t, [key]));
    toasts[key]?.onClose?.();
  });
  return /*#__PURE__*/_jsx(ToastContext.Provider, {
    value: contextValue,
    children: /*#__PURE__*/_jsxs(ToastPrimitive.Provider, {
      swipeDirection: "right",
      children: [children, Object.values(toasts).map(props => /*#__PURE__*/_jsx(Toast, {
        ...props,
        onClose: () => handleClose(props.key)
      })), /*#__PURE__*/_jsx(Portal, {
        container: container,
        children: /*#__PURE__*/_jsx(ToastViewport, {})
      })]
    })
  });
}
if (process.env.NODE_ENV !== "production") {
  ToastProvider.displayName = "ToastProvider";
}